//
//  ViewController.swift
//  Maps
//
//  Created by Holger Hinzberg on 06.02.16.
//  Copyright © 2016 Holger Hinzberg. All rights reserved.
//

import UIKit
import MapKit

class ViewController: UIViewController,
    CLLocationManagerDelegate, MKMapViewDelegate
{
    @IBOutlet var latitudeLabel: UILabel!
    @IBOutlet var longitudeLabel: UILabel!
    @IBOutlet var timestampLabel: UILabel!
    @IBOutlet var locationLabel: UILabel!
    @IBOutlet var mapView: MKMapView!
    
    var locationManager:CLLocationManager?
    
    override func viewDidLoad()
    {
        super.viewDidLoad()
    
        // Den CLLocationManager initialisieren
        // und konfigurieren
        self.locationManager = CLLocationManager()
        self.locationManager?.desiredAccuracy
            = kCLLocationAccuracyNearestTenMeters
        self.locationManager?.distanceFilter = 50
        self.locationManager?.delegate = self
        self.locationManager?.requestWhenInUseAuthorization()
        
        // Die Benachrichtigungen nur dann starten
        // wenn dies zuvor vom Anwender erlaubt wurde
        self.locationManager?.requestWhenInUseAuthorization()
        if CLLocationManager.locationServicesEnabled()
        {
            self.locationManager?.startUpdatingLocation()
        }

        self.mapView.showsUserLocation = true
        self.mapView.mapType = MKMapType.Hybrid
        
        // Der View Controller wird zum Delegate des MKMapView
        self.mapView.delegate = self
        
        //self.geocodeAddressSample()
        self.addAnnoations()
    }
    
    func addAnnoations()
    {
        let pointMuseum = MKPointAnnotation()
        pointMuseum.title = "Römisch-Germanisches Museum"
        pointMuseum.subtitle = "Im Museum. Das war toll!"
        pointMuseum.coordinate
        = CLLocationCoordinate2DMake(50.9406, 6.9586)
        // Dem MKMapView die Annotation hinzufügen
        self.mapView.addAnnotation(pointMuseum)
        
        let pointStation = MKPointAnnotation()
        pointStation.title = "Köln Hauptbahnhof"
        pointStation.subtitle = "Hier bin ich angekommen"
        pointStation.coordinate
            = CLLocationCoordinate2DMake(50.9428, 6.9576)
        self.mapView.addAnnotation(pointStation)
    }
    
    func mapView(mapView: MKMapView, viewForAnnotation annotation: MKAnnotation) -> MKAnnotationView?
    {
        // Ist dies die eigene Position?
        if annotation is MKUserLocation
        {
            return nil
        }
        
        let identifier = "MyAnnotation"
        
        var annotationView = self.mapView
            .dequeueReusableAnnotationViewWithIdentifier(identifier)
        
        if annotationView == nil
        {
            annotationView = MKAnnotationView(
                annotation: annotation, reuseIdentifier: identifier)
            
            // Zur Anziege der Annotation eine Grafik verwenden
            annotationView?.image = UIImage(named: "customPin.png")
            // Die Annotation korrekt positionieren
            annotationView?.centerOffset = CGPointMake(0, -24.5)
        }
        
        // Die Anmerkungen können ausgewählt werden,
        // um dort Informationen anzuzeigen
        annotationView?.canShowCallout = true
        
        // Links im Fähnchen soll eine Grafik angezeigt werden.
        let image = UIImage(named: "info.png")
        let leftCalloutView = UIImageView(image: image)
        leftCalloutView.frame = CGRectMake(0, 0, 30, 30)
        annotationView?.leftCalloutAccessoryView
          = leftCalloutView
        
        // Der rechte Accessoryx View zeig einen
        // System-Button. UIButton ist ebenfalls 
        // von UIView abgeleitet
        annotationView?.rightCalloutAccessoryView
          = UIButton(type: UIButtonType.InfoDark)
        
        return annotationView
    }
    
    func mapView(mapView: MKMapView,
        annotationView view: MKAnnotationView,
        calloutAccessoryControlTapped control: UIControl)
    {
        let annotation = view.annotation
        print(annotation?.title)
        print(annotation?.subtitle)
    }
    
    /*
    func mapView(mapView: MKMapView, viewForAnnotation annotation: MKAnnotation) -> MKAnnotationView?
    {
        // Ist dies die eigene Position?
        if annotation is MKUserLocation
        {
         return nil
        }
        
        let identifier = "MyAnnotation"
        
        var annotationView = self.mapView
            .dequeueReusableAnnotationViewWithIdentifier(identifier)
            as? MKPinAnnotationView
        
        // Einen neuen View anlegen, wenn kein 
        // wiederverwendbarer View zur Verfügung steht
        if annotationView == nil
        {
            annotationView = MKPinAnnotationView(
                annotation: annotation, reuseIdentifier: identifier)
        }
    
        if annotation is MKUserLocation
        {
            annotationView?.pinTintColor = UIColor.greenColor()
        }
        
        // Die Anmerkungen können ausgewählt werden,
        // um dort Informationen anzuzeigen
        annotationView?.canShowCallout = true
        
        return annotationView
    }
    */
    

    func locationManager(manager: CLLocationManager,
        didUpdateToLocation newLocation: CLLocation,
        fromLocation oldLocation: CLLocation)
    {
        // Die Konfiguration der anzuzeigenden Region
        // für das MKMapView-Steuerelement
        let span = MKCoordinateSpanMake(0.005, 0.005)
        let region = MKCoordinateRegionMake(newLocation.coordinate, span)
        self.mapView.region = region
        
        // Wann wurden die Positionsinformationen empfangen?
        let timestamp = newLocation.timestamp
        let formatter = NSDateFormatter()
        formatter.dateFormat = "dd.MM.yyyy hh:mm:ss"
        let formattedTimestamp
        = formatter.stringFromDate(timestamp)
        self.timestampLabel.text = formattedTimestamp
        
        // Der Längengrad der aktuellen Position
        let longitude = newLocation.coordinate.longitude
        self.longitudeLabel.text =
        String(format: "Längengrad: %.6f", longitude)
        
        // Der Breitengrad der aktuellen Position
        let latitude = newLocation.coordinate.latitude
        self.latitudeLabel.text =
        String(format: "Breitengrad: %.6f", latitude)
        
        // Einen Geocoder anlegen und initialisieren
        let geoCoder = CLGeocoder()
        geoCoder.reverseGeocodeLocation(newLocation)
            {
                (placemarks, error) in
                print("completionHandler aufgerufen")
                
                if error != nil
                {
                    print("Fehler: \(error!.localizedDescription)")
                }
                else if let places = placemarks
                {
                    if places.count > 0
                    {
                        let location = String(format: "%@ %@, %@ %@",
                            places[0].thoroughfare!,
                            places[0].subThoroughfare!,
                            places[0].postalCode!,
                            places[0].locality!)
                        self.locationLabel.text = location
                    }
                }
        }
    }
    
    
    /*
    func locationManager(manager: CLLocationManager,
        didUpdateToLocation newLocation: CLLocation,
        fromLocation oldLocation: CLLocation)
    {
        // Wann wurden die Positionsinformationen empfangen?
        let timestamp = newLocation.timestamp
        
        // Der Längengrad der aktuellen Position
        let longitude = newLocation.coordinate.longitude
        
        // Der Breitengrad der aktuellen Position
        let latitude = newLocation.coordinate.latitude
        
        // Die horizontale und vertikale Genauigkeit
        let horizontal = newLocation.horizontalAccuracy
        let vertical = newLocation.verticalAccuracy
        
        let formatter = NSDateFormatter()
        formatter.dateFormat = "dd.MM.yyyy hh:mm:ss"
        let formattedTimestamp
          = formatter.stringFromDate(timestamp)
        
        print("Empfangen am: \(formattedTimestamp)")
        print(String(format: "Längengrad: %.2f", longitude))
        print(String(format: "Breitengrad: %.2f", latitude))
        print("Horizontale Genauigkeit \(horizontal) Meter")
        print("Vertikale Genauigkeit \(vertical) Meter")
        
        // Optional:
        // Keine weiteren Positionsinformationen anfordern
        self.locationManager?.stopUpdatingLocation()
        
        // Einen Geocoder anlegen und initialisieren
        let geoCoder = CLGeocoder()
        geoCoder.reverseGeocodeLocation(newLocation)
        {
            (placemarks, error) in
                print("completionHandler aufgerufen")
            
                if error != nil
                {
                    print("Fehler: \(error!.localizedDescription)")
                }
                else if let places = placemarks
                {
                    // Ausgabe der Informationen
                    for place in places
                    {
                        print(place.thoroughfare)
                        print(place.subThoroughfare)
                        print(place.postalCode)
                        print(place.locality)
                        print(place.administrativeArea)
                        print(place.country)
                    }
                }
           }
    }
    */
    
    func geocodeAddressSample()
    {
        // Die gesuchte Adresse wird 
        // als Zeichenkette angegeben
        let place = "Dortmund Florianstraße 1"
        
        let geoCoder = CLGeocoder()
        geoCoder.geocodeAddressString(place)
        {
            (placemarks, error) in
            print("completionHandler aufgerufen")
            
            if error != nil
            {
                print("Fehler: \(error!.localizedDescription)")
            }
            else if let places = placemarks
            {
                // Ausgabe der Informationen
                for place in places
                {
                    print(place.thoroughfare)
                    print(place.subThoroughfare)
                    print(place.postalCode)
                    print(place.locality)
                    print(place.administrativeArea)
                    print(place.country)
                    
                    // Längengrad und Breitengrad
                    if let location = place.location
                    {
                        let longitude = String(format:"%+.6f",
                            location.coordinate.longitude)
                    
                        let latitude = String(format:"%+.6f",
                            location.coordinate.latitude)
                        
                        print("Längengrad: \(longitude)")
                        print("Breitengrad: \(latitude)")
                    }
                }
            }
        }
    }

}

